local addonName, addonTable = ...
local AceConfig = _G.LibStub("AceConfig-3.0")
local AceConfigDialog = _G.LibStub("AceConfigDialog-3.0")
local E = addonTable.E
local L = addonTable.L
local wipe = _G.wipe
local pairs = _G.pairs
local string_match =  _G.string.match
local strtrim = _G.strtrim
local print = _G.print
local GetTalentInfoBySpecialization = _G.GetTalentInfoBySpecialization
local CreateFrame = _G.CreateFrame

local optionsDB
E.config = {}

local function getTalentsForSpec(spec, tier)
    local t = {}
    for i=1, 3 do
        local id, name = GetTalentInfoBySpecialization(spec, tier, i)
        t[i] = name
    end
    return t
end

local function getSpecTable()
    local specTable = {}
    specTable[0] = L["Don't switch"]
    for i= 1, #E.specTable do
        specTable[i] = E.specTable[i].name
    end
    return specTable
end

local function buildSpecOptions(equipmentSet)
    local argTable = {}
    for i=1, 7 do
        argTable["tier"..i] = {
            type = "select",
            name = L["Tier "..i],
            values = getTalentsForSpec(equipmentSet.specialization or 1, i),
            set = function(self, value)
                equipmentSet.talents[i] = value
            end,
            get = function()
                return equipmentSet.talents[i]
            end,
        }
    end
    return argTable
end

function E.config:showStandaloneOptions()
    if not AceConfigDialog.OpenFrames[addonName] then
        AceConfigDialog:Open(addonName)
    end
end

function E.config:refreshOptions()
    E:gatherEquipmentSets()
    -- todo: parse options table (memory leak)
    self:buildOptionsTable()
end

function E.config:buildOptionsTable()
    if self.optionsTable then
        wipe(self.optionsTable.args)
    else
        self.optionsTable = {}
        self.optionsTable.type = "group"
        self.optionsTable.args = {}
    end
    local args = self.optionsTable.args
    args["GeneralMultiSpecOptions"] = {
        name = L["General options"],
        type = "group",
        order = 1,
        args = {
            toggle = {
                name = L["Hide character frame button"],
                --desc = L["Will hide the Character frame button."],
                type = "toggle",
                --width = "half",
                get = function()
                    return optionsDB.hideInCharFrame
                end,
                set = function(self, value)
                    if value then
                        E.ui:hide()
                    else
                        E.ui:show()
                    end
                    optionsDB.hideInCharFrame = value
                end,
            }
        }
    }
    for k, v in pairs(E.equipmentSets) do
        args[k] = {
            name = k,
            type = "group",
            args = {
                select = {
                    name = L["Specialization"],
                    type = "select",
                    values = getSpecTable(),
                    set = function(self, value)
                        v.specialization = value
                        E.config:refreshOptions()
                    end,
                    get = function()
                        return v.specialization
                    end,
                },
                toggle = {
                    name = L["Switch talents"],
                    desc = L["If you enable this, talents for this specialization will be switched to them below if you are in a resting area."],
                    type = "toggle",
                    --width = "half",
                    get = function()
                        return v.switchTalents
                    end,
                    set = function(self, value)
                        if value then
                            v.switchTalents = true
                        else
                            v.switchTalents = false
                        end
                        E.config:refreshOptions()
                    end,
                },
                talents = {
                    name = L["Talents"],
                    type = "group",
                    disabled = not v.switchTalents,
                    inline = true,
                    args = buildSpecOptions(v)
                },
                button = {
                    name = L["Activate"],
                    type = "execute",
                    order = -1,
                    --image = v.texture or 134400,
                    width = "full",
                    func = function(info)
                        E:changeTalentsAndEquipSet(info[1])
                    end,
                },
            },
        }
    end
end

function E.config.hideInCharFrame()
    return optionsDB.hideInCharFrame
end

function E.config:init()
    if not _G.MultiSpecDB.options then
        _G.MultiSpecDB.options = {}
    end
    optionsDB = _G.MultiSpecDB.options
    self.frame = CreateFrame("Frame")
    self.frame:RegisterEvent("EQUIPMENT_SETS_CHANGED")
    self.frame:SetScript("OnEvent", function(self, event, ...)
        if event == "EQUIPMENT_SETS_CHANGED" then
            E.config:refreshOptions()
        end
    end)
    self:buildOptionsTable()
    AceConfig:RegisterOptionsTable("MultiSpec", self.optionsTable)
    AceConfigDialog:AddToBlizOptions("MultiSpec")
end

for i, v in pairs({"ms", "multispec"}) do
	_G["SLASH_MULTISPEC"..i] = "/"..v
end

local function print_help()
    print("type /ms for options or")
    print("/ms equip [set] to change your equipment set")
end

function _G.SlashCmdList.MULTISPEC(msg, editbox)
    if strtrim(msg) ~= "" then
        local set = string_match(msg, "^equip%s+(%S+)")
        if set then
            E:changeTalentsAndEquipSet(set)
        else
            print_help()
        end
    else
        E.config:showStandaloneOptions()
    end
end
