local addonName, addonTable = ...

local E = {}
-- Localization stuff
local L = {}
addonTable.L = L
local function localizationDefault(L, key)
    return key;
end

_G.setmetatable(L, {__index=localizationDefault});
E.L = L
-- end localization

-- Globals
local wipe = _G.wipe
local pairs = _G.pairs
local UseEquipmentSet = _G.UseEquipmentSet
local GetNumEquipmentSets = _G.GetNumEquipmentSets
local GetSpecialization = _G.GetSpecialization
local GetSpecializationInfo = _G.GetSpecializationInfo
local GetTalentInfo = _G.GetTalentInfo
local LearnTalent = _G.LearnTalent
local CreateFrame = _G.CreateFrame
local GetNumSpecializations = _G.GetNumSpecializations
local GetEquipmentSetInfo = _G.GetEquipmentSetInfo
local SetSpecialization = _G.SetSpecialization
local IsResting = _G.IsResting
local GetSpellInfo = _G.GetSpellInfo
local UnitBuff = _G.UnitBuff
local GetTime = _G.GetTime
--

local RESPEC_TIMEOUT = 5.5

E.events = {}
E.specTable = {}
E.equipmentSets = {}
addonTable.E = E

_G.MultiSpec = E

local tome = GetSpellInfo(227041)

function E.events:PLAYER_ENTERING_WORLD(...)
    E:Init()
end

function E.events:PLAYER_REGEN_ENABLED(...)
    E.inCombat = false
end

function E.events:PLAYER_REGEN_DISABLED(...)
    E.inCombat = true
end

function E.events:ACTIVE_TALENT_GROUP_CHANGED(now, before)
    if GetTime() - E.timeout < RESPEC_TIMEOUT then
        E:activateSet(E.activeSet)
    end
    E:stopListenforSpecChange()
end

function E:Load()
    E.frame = CreateFrame("Frame")
    E.frame:RegisterEvent("PLAYER_ENTERING_WORLD")
    E.frame:SetScript("OnEvent", function(self, event, ...)
        E.events[event](self, ...)
    end)
end

function E:listenForSpecChange()
    E.frame:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
end

function E:stopListenforSpecChange()
    E.frame:UnregisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
end

function E:buildSpecTable()
    local n = GetNumSpecializations()
    for i=1, n do
        local id, name, description, icon, background, role, primaryStat = GetSpecializationInfo(i)
        self.specTable[i] = {}
        self.specTable[i].name = name
        self.specTable[i].description = description
        self.specTable[i].icon = icon
        self.specTable[i].background = background
        self.specTable[i].role = role
        self.specTable[i].primaryStat = primaryStat
    end
end

function E:Init()
    if not self.firstInit then
        self.firstInit = true
        if _G.MultiSpecDB == nil then
            _G.MultiSpecDB = {}
            _G.MultiSpecDB.equipmentSets = E.equipmentSets
            _G.MultiSpecDB.options = {}
        else
            E.equipmentSets = _G.MultiSpecDB.equipmentSets
        end
        self:buildSpecTable()
        self:gatherEquipmentSets()
        self.frame:RegisterEvent("PLAYER_REGEN_ENABLED")
        self.frame:RegisterEvent("PLAYER_REGEN_DISABLED")
        self.config:init()
        self.ui:init()
        if not self.config.hideInCharFrame() then
            self.ui:show()
        else
            self.ui:hide()
        end
        self.databroker:init()
        self.elvui:init()
    end
end

function E:gatherEquipmentSets()
    -- Equipmentsets may have changed since last scan, so we have to do a rescan
    local esCount = GetNumEquipmentSets()
    -- add new equipmentsets
    for i=1, esCount do
        local name, texture, setIndex, isEquipped, totalItems, equippedItems, inventoryItems, missingItems, ignoredSlots = GetEquipmentSetInfo(i)
        if not E.equipmentSets[name] then
            E.equipmentSets[name] = {}
            E.equipmentSets[name].switchTalents = false
            E.equipmentSets[name].talents = {1, 1, 1, 1, 1, 1, 1}
            E.equipmentSets[name].specialization = 0
        end
        E.equipmentSets[name].name = name
        E.equipmentSets[name].texture = texture
        E.equipmentSets[name].setIndex = setIndex
        E.equipmentSets[name].isEquipped = isEquipped
        E.equipmentSets[name].totalItems = totalItems
        E.equipmentSets[name].equippedItems = equippedItems
        E.equipmentSets[name].inventoryItems = inventoryItems
        E.equipmentSets[name].missingItems = missingItems
        E.equipmentSets[name].ignoredSlots = ignoredSlots
        E.equipmentSets[name].new = true
    end
    --delete no longer available equipmentsets
    for k, v in pairs(E.equipmentSets) do
        if E.equipmentSets[k].new then
            E.equipmentSets[k].new = false
        else
            E.equipmentSets[k] = nil
        end
    end
end

function E:equipSet(set)
    if not UseEquipmentSet(set) then
        E.error:equipSetFailed(set)
    end
end

function E:changeTalents(set)
    if E.equipmentSets[set].switchTalents then
        for k,v in pairs(E.equipmentSets[set].talents) do
            local id = GetTalentInfo(k,v,1)
            LearnTalent(id)
        end
    end
end

function E:activateSet(set)
    if (IsResting() or UnitBuff("player", tome)) then
        E:changeTalents(set)
    else
        if E.equipmentSets[set].switchTalents then
            E.error:notInRestingArea()
        end
    end
    E:equipSet(set)
end

function E:changeTalentsAndEquipSet(set)
    local spec = GetSpecialization()
    if E.equipmentSets[set] then
        if E.equipmentSets[set].missingItems == 0 then
            if not E.inCombat then
                if E.equipmentSets[set].specialization ~= 0 and spec ~= E.equipmentSets[set].specialization then
                    E.activeSet = set
                    self:listenForSpecChange()
                    E.timeout = GetTime()
                    SetSpecialization(E.equipmentSets[set].specialization)
                else
                    E.activeSet = set
                    E:activateSet(set)
                end
            else
                E.error:inCombat()
            end
        else
            E.error:itemsForEquipmentSetMissing(set)
        end
    else
        E.error:equipmentSetNotFound(set)
    end
end

E:Load()
